/*
 * Project name:
     RTC_Read (Demonstration on Working with the RTC Module)
 * Copyright:
     (c) MikroElektronika, 2006.
 * 29/06/2014 : réadaptation et association du code lecture et écriture de la PCF8583 et conversion de l'exemple project version ancien MikroC en version MikroC pro v6.0.0.0
 
 * Description:
      This project is simple demonstration how to Read and Write date and time from
      PCF8583 RTC (real-time clock). The code can be used with any MCU that has
      MSSP module at PORTC.
      The offset from Y2000 is stored in the PCF8583 RAM at address 0x10.
      The time format is 24h and weekdays are not used.
 * Test configuration:
     MCU:             PIC16F887
     Dev.Board:       EasyPIC5
     Oscillator:      HS, 8.000 MHz
     Ext. Modules:    RTC on PORTC
     SW:              mikroC Pro v6.0.0.0
 * NOTES:
     - Be sure to switch the portC LEDs OFF
     - For proper I2C communication, PORTC must be in the pull-up mode
     - In order to use the example, address pin A0 of PCF8583 must be set to 0V.
       (on mikroElektronika's RTC module this is done by default)
 */
struct TTime
{
  char year, month, day, hours, minutes, seconds;
} TimeRW;

char yearmod4, byteRead;

void Display_Time() 
{
 char txt[11];
   // output values to LCD display
   txt[0] = (TimeRW.day / 10) + 48;
   txt[1] = (TimeRW.day % 10) + 48;
   txt[2] = '/';
   txt[3] = (TimeRW.month / 10) + 48;
   txt[4] = (TimeRW.month % 10) + 48;
   txt[5] = '/';
   txt[6] = '2';
   txt[7] = (TimeRW.year / 100) + 48;
   txt[8] = ((TimeRW.year % 100) / 10) + 48;
   txt[9] = (TimeRW.year % 10)  + 48;
   txt[10] = 0;  // null to terminate the string
   Lcd_Out(1,7,txt);

   txt[0] = (TimeRW.hours / 10) + 48;
   txt[1] = (TimeRW.hours % 10) + 48;
   txt[2] = ':';
   txt[3] = (TimeRW.minutes / 10) + 48;
   txt[4] = (TimeRW.minutes % 10) + 48;
   txt[5] = ':';
   txt[6] = (TimeRW.seconds / 10) + 48;
   txt[7] = (TimeRW.seconds % 10) + 48;
   txt[8] = 0;  // null to terminate the string
   Lcd_Out(2,7,txt);
}

void WriteTime()
{
   TimeRW.year    = 7;    // 2007 (offset from 2000)
   TimeRW.month   = 12;
   TimeRW.day     = 31;
   TimeRW.hours   = 23;
   TimeRW.minutes = 59;
   TimeRW.seconds = 40;

   I2C1_Start();            // issue start signal
   I2C1_Wr(0xA0);          // address PCF8583
   I2C1_Wr(0);            // start from word at address 0 (configuration word)
   I2C1_Wr(0x80);          // write 0x80 to config. (pause counter...)
   I2C1_Wr(0);            // write 0 to cents word
   I2C1_Wr(((TimeRW.seconds/10)<<4) + (TimeRW.seconds%10)); // write seconds word
   I2C1_Wr(((TimeRW.minutes/10)<<4) + (TimeRW.minutes%10)); // write minutes word
   I2C1_Wr(((TimeRW.hours/10)<<4) + (TimeRW.hours%10)); // write hours word
   I2C1_Wr(((TimeRW.year%4)<<6) + ((TimeRW.day/10)<<4) +(TimeRW.day%10));
   I2C1_Wr(((TimeRW.month/10)<<4) + (TimeRW.month%10));  // write weekday/month
   I2C1_Stop();             // issue stop signal

   I2C1_Start();            // issue start signal
   I2C1_Wr(0xA0);          // address PCF8583
   I2C1_Wr(0x10);          // start from word at address 0x10
   I2C1_Wr(TimeRW.year);         // write year to RAM
   I2C1_Stop();             // issue stop signal

   I2C1_Start();            // issue start signal
   I2C1_Wr(0xA0);          // address PCF8583
   I2C1_Wr(0);            // start from word at address 0
   I2C1_Wr(0);            // write 0 to config word (enable counting)
   I2C1_Stop();             // issue stop signal
}

void ReadTime()
{
 char updateYear;

   updateYear = 0;

   I2C1_Start();                // issue start signal
   I2C1_Wr(0xA0);               // address PCF8583
   I2C1_Wr(2);                  // first word address
   I2C1_Repeated_Start();       // issue repeated start signal
   I2C1_Wr(0xA1);               // address PCF8583 for reading R/W=1

   byteRead = I2C1_Rd(1);       // read seconds byte
   TimeRW.seconds = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform seconds
   //while(I2C1_Is_Idle() == 0)  asm nop;    // this lines are not neccessary in new versions
                      // starting from mikroC 6.0.0.1

   byteRead = I2C1_Rd(1);     // read minutes byte
   TimeRW.minutes = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform minutes
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(1);     // read hours byte
   TimeRW.hours = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform hours
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(1);     // read year/day byte
   TimeRW.day = ((byteRead & 0b00110000) >> 4)*10 + (byteRead & 0x0F); // transform day
   yearmod4 = (byteRead & 0b11000000) >> 6;   // get year mod 4 from RTC
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(0);     // read weekday/month byte
   TimeRW.month = ((byteRead & 0b00010000) >> 4)*10 + (byteRead & 0x0F); // transform month
   //while (I2C1_Is_Idle() == 0) asm nop;
   I2C1_Stop();

   I2C1_Start();                 // issue start signal
   I2C1_Wr(0xA0);               // address PCF8583
   I2C1_Wr(0x10);               // first word address
   I2C1_Repeated_Start();        // issue repeated start signal
   I2C1_Wr(0xA1);               // address PCF8583 for reading R/W=1

   byteRead = I2C1_Rd(0);     // read year
   if(yearmod4 != byteRead % 4 )
   { // check if year is incremented in RTC
       byteRead++;  // in this case the new value should be written to RTC RAM at address 16 (0x10)
       updateYear = 1;
   }
   TimeRW.year = byteRead;

   //while (I2C1_Is_Idle() == 0)  asm nop;
   I2C1_Stop();

   if(updateYear > 0)
   {
      I2C1_Start();            // issue start signal
      I2C1_Wr(0xA0);          // address PCF8583
      I2C1_Wr(0x10);          // start from word at address 0x10
      I2C1_Wr(TimeRW.year);         // write year to RAM
      I2C1_Stop();             // issue stop signal
    }
}

// LCD module connections
sbit LCD_EN at RB5_bit;
sbit LCD_RS at RB4_bit;

sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_EN_Direction at TRISB5_bit;  // orange
sbit LCD_RS_Direction at TRISB4_bit;  // bleu

sbit LCD_D4_Direction at TRISB0_bit;  // vert
sbit LCD_D5_Direction at TRISB1_bit;  // jaune
sbit LCD_D6_Direction at TRISB2_bit;  // gris
sbit LCD_D7_Direction at TRISB3_bit;  // rose

// RW = GND

// End LCD module connections

void MainInit()
{
  ANSEL = 0;
  ANSELH = 0;
  
  Lcd_Init();                        // Initialize LCD

  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out(1,1,"Date:   /  /");
  Lcd_Out(2,1,"Time:   :  :");

  I2C1_Init(10000);      // initialize full master mode I2C
  
  WriteTime();
}



void main()
{
   MainInit();

   while(1)
   {
     ReadTime();
     Display_Time();
     Delay_ms(1000);
   }
}